@@ -1,11 +1,8 @@
 //! Feature resolver.
 //!
 //! This is a new feature resolver that runs independently of the main
-//! dependency resolver. It is intended to make it easier to experiment with
-//! new behaviors. When `-Zfeatures` is not used, it will fall back to using
-//! the original `Resolve` feature computation. With `-Zfeatures` enabled,
-//! this will walk the dependency graph and compute the features using a
-//! different algorithm.
+//! dependency resolver. It is enabled when the user specifies `resolver =
+//! "2"` in `Cargo.toml`.
 //!
 //! One of its key characteristics is that it can avoid unifying features for
 //! shared dependencies in some situations. See `FeatureOpts` for the
@@ -45,7 +42,8 @@ use crate::core::resolver::{Resolve, ResolveBehavior};
 use crate::core::{FeatureValue, PackageId, PackageIdSpec, PackageSet, Workspace};
 use crate::util::interning::InternedString;
 use crate::util::CargoResult;
-use std::collections::{BTreeSet, HashMap, HashSet};
+use anyhow::bail;
+use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::rc::Rc;
 
 /// Map of activated features.
@@ -61,11 +59,11 @@ pub struct ResolvedFeatures {
     ///
     /// The value is the `name_in_toml` of the dependencies.
     activated_dependencies: ActivateMap,
-    /// This is only here for legacy support when `-Zfeatures` is not enabled.
+    /// This is only here for legacy support when the new resolver is not enabled.
     ///
     /// This is the set of features enabled for each package.
     legacy_features: Option<HashMap<PackageId, Vec<InternedString>>>,
-    /// This is only here for legacy support when `-Zfeatures` is not enabled.
+    /// This is only here for legacy support when the new resolver is not enabled.
     ///
     /// This is the set of optional dependencies enabled for each package.
     legacy_dependencies: Option<HashMap<PackageId, HashSet<InternedString>>>,
@@ -74,8 +72,8 @@ pub struct ResolvedFeatures {
 
 /// Options for how the feature resolver works.
 #[derive(Default)]
-struct FeatureOpts {
-    /// -Zfeatures is enabled, use new resolver.
+pub struct FeatureOpts {
+    /// Use the new resolver instead of the old one.
     new_resolver: bool,
     /// Build deps and proc-macros will not share share features with other dep kinds.
     decouple_host_deps: bool,
@@ -126,7 +124,7 @@ impl FeaturesFor {
 }
 
 impl FeatureOpts {
-    fn new(
+    pub fn new(
         ws: &Workspace<'_>,
         has_dev_units: HasDevUnits,
         force_all_targets: ForceAllTargets,
@@ -147,7 +145,7 @@ impl FeatureOpts {
                     }
                     "compare" => opts.compare = true,
                     "ws" => unimplemented!(),
-                    s => anyhow::bail!("-Zfeatures flag `{}` is not supported", s),
+                    s => bail!("-Zfeatures flag `{}` is not supported", s),
                 }
             }
             Ok(())
@@ -183,47 +181,110 @@ impl FeatureOpts {
         }
         Ok(opts)
     }
+
+    /// Creates a new FeatureOpts for the given behavior.
+    pub fn new_behavior(behavior: ResolveBehavior, has_dev_units: HasDevUnits) -> FeatureOpts {
+        match behavior {
+            ResolveBehavior::V1 => FeatureOpts::default(),
+            ResolveBehavior::V2 => FeatureOpts {
+                new_resolver: true,
+                decouple_host_deps: true,
+                decouple_dev_deps: has_dev_units == HasDevUnits::No,
+                ignore_inactive_targets: true,
+                compare: false,
+            },
+        }
+    }
 }
 
 /// Features flags requested for a package.
+///
+/// This should be cheap and fast to clone, it is used in the resolver for
+/// various caches.
+///
+/// This is split into enum variants because the resolver needs to handle
+/// features coming from different places (command-line and dependency
+/// declarations), but those different places have different constraints on
+/// which syntax is allowed. This helps ensure that every place dealing with
+/// features is properly handling those syntax restrictions.
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
-pub struct RequestedFeatures {
-    pub features: FeaturesSet,
+pub enum RequestedFeatures {
+    /// Features requested on the command-line with flags.
+    CliFeatures(CliFeatures),
+    /// Features specified in a dependency declaration.
+    DepFeatures {
+        /// The `features` dependency field.
+        features: FeaturesSet,
+        /// The `default-features` dependency field.
+        uses_default_features: bool,
+    },
+}
+
+/// Features specified on the command-line.
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+pub struct CliFeatures {
+    /// Features from the `--features` flag.
+    pub features: Rc<BTreeSet<FeatureValue>>,
+    /// The `--all-features` flag.
     pub all_features: bool,
+    /// Inverse of `--no-default-features` flag.
     pub uses_default_features: bool,
 }
 
-impl RequestedFeatures {
-    /// Creates a new RequestedFeatures from the given command-line flags.
+impl CliFeatures {
+    /// Creates a new CliFeatures from the given command-line flags.
     pub fn from_command_line(
         features: &[String],
         all_features: bool,
         uses_default_features: bool,
-    ) -> RequestedFeatures {
-        RequestedFeatures {
-            features: Rc::new(RequestedFeatures::split_features(features)),
+    ) -> CargoResult<CliFeatures> {
+        let features = Rc::new(CliFeatures::split_features(features));
+        // Some early validation to ensure correct syntax.
+        for feature in features.iter() {
+            match feature {
+                // Maybe call validate_feature_name here once it is an error?
+                FeatureValue::Feature(_) => {}
+                FeatureValue::Dep { .. }
+                | FeatureValue::DepFeature {
+                    dep_prefix: true, ..
+                } => {
+                    bail!(
+                        "feature `{}` is not allowed to use explicit `dep:` syntax",
+                        feature
+                    );
+                }
+                FeatureValue::DepFeature { dep_feature, .. } => {
+                    if dep_feature.contains('/') {
+                        bail!("multiple slashes in feature `{}` is not allowed", feature);
+                    }
+                }
+            }
+        }
+        Ok(CliFeatures {
+            features,
             all_features,
             uses_default_features,
-        }
+        })
     }
 
-    /// Creates a new RequestedFeatures with the given `all_features` setting.
-    pub fn new_all(all_features: bool) -> RequestedFeatures {
-        RequestedFeatures {
+    /// Creates a new CliFeatures with the given `all_features` setting.
+    pub fn new_all(all_features: bool) -> CliFeatures {
+        CliFeatures {
             features: Rc::new(BTreeSet::new()),
             all_features,
             uses_default_features: true,
         }
     }
 
-    fn split_features(features: &[String]) -> BTreeSet<InternedString> {
+    fn split_features(features: &[String]) -> BTreeSet<FeatureValue> {
         features
             .iter()
             .flat_map(|s| s.split_whitespace())
             .flat_map(|s| s.split(','))
             .filter(|s| !s.is_empty())
             .map(InternedString::new)
-            .collect::<BTreeSet<InternedString>>()
+            .map(FeatureValue::new)
+            .collect()
     }
 }
 
@@ -285,15 +346,75 @@ impl ResolvedFeatures {
             if let Some(fs) = self.activated_features.get(&(pkg_id, is_build)) {
                 Ok(fs.iter().cloned().collect())
             } else {
-                anyhow::bail!("features did not find {:?} {:?}", pkg_id, is_build)
+                bail!("features did not find {:?} {:?}", pkg_id, is_build)
             }
         }
     }
+
+    /// Compares the result against the original resolver behavior.
+    ///
+    /// Used by `cargo fix --edition` to display any differences.
+    pub fn compare_legacy(&self, legacy: &ResolvedFeatures) -> FeatureDifferences {
+        let legacy_features = legacy.legacy_features.as_ref().unwrap();
+        let features = self
+            .activated_features
+            .iter()
+            .filter_map(|((pkg_id, for_host), new_features)| {
+                let old_features = match legacy_features.get(pkg_id) {
+                    Some(feats) => feats.iter().cloned().collect(),
+                    None => BTreeSet::new(),
+                };
+                // The new resolver should never add features.
+                assert_eq!(new_features.difference(&old_features).next(), None);
+                let removed_features: BTreeSet<_> =
+                    old_features.difference(new_features).cloned().collect();
+                if removed_features.is_empty() {
+                    None
+                } else {
+                    Some(((*pkg_id, *for_host), removed_features))
+                }
+            })
+            .collect();
+        let legacy_deps = legacy.legacy_dependencies.as_ref().unwrap();
+        let optional_deps = self
+            .activated_dependencies
+            .iter()
+            .filter_map(|((pkg_id, for_host), new_deps)| {
+                let old_deps = match legacy_deps.get(pkg_id) {
+                    Some(deps) => deps.iter().cloned().collect(),
+                    None => BTreeSet::new(),
+                };
+                // The new resolver should never add dependencies.
+                assert_eq!(new_deps.difference(&old_deps).next(), None);
+                let removed_deps: BTreeSet<_> = old_deps.difference(new_deps).cloned().collect();
+                if removed_deps.is_empty() {
+                    None
+                } else {
+                    Some(((*pkg_id, *for_host), removed_deps))
+                }
+            })
+            .collect();
+        FeatureDifferences {
+            features,
+            optional_deps,
+        }
+    }
+}
+
+/// Map of differences.
+///
+/// Key is `(pkg_id, for_host)`. Value is a set of features or dependencies removed.
+pub type DiffMap = BTreeMap<(PackageId, bool), BTreeSet<InternedString>>;
+
+/// Differences between resolvers.
+pub struct FeatureDifferences {
+    pub features: DiffMap,
+    pub optional_deps: DiffMap,
 }
 
 pub struct FeatureResolver<'a, 'cfg> {
     ws: &'a Workspace<'cfg>,
-    target_data: &'a RustcTargetData,
+    target_data: &'a RustcTargetData<'cfg>,
     /// The platforms to build for, requested by the user.
     requested_targets: &'a [CompileKind],
     resolve: &'a Resolve,
@@ -331,19 +452,17 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     /// with the result.
     pub fn resolve(
         ws: &Workspace<'cfg>,
-        target_data: &RustcTargetData,
+        target_data: &RustcTargetData<'cfg>,
         resolve: &Resolve,
         package_set: &'a PackageSet<'cfg>,
-        requested_features: &RequestedFeatures,
+        cli_features: &CliFeatures,
         specs: &[PackageIdSpec],
         requested_targets: &[CompileKind],
-        has_dev_units: HasDevUnits,
-        force_all_targets: ForceAllTargets,
+        opts: FeatureOpts,
     ) -> CargoResult<ResolvedFeatures> {
         use crate::util::profile;
         let _p = profile::start("resolve features");
 
-        let opts = FeatureOpts::new(ws, has_dev_units, force_all_targets)?;
         if !opts.new_resolver {
             // Legacy mode.
             return Ok(ResolvedFeatures {
@@ -368,7 +487,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             track_for_host,
             deferred_weak_dependencies: HashMap::new(),
         };
-        r.do_resolve(specs, requested_features)?;
+        r.do_resolve(specs, cli_features)?;
         log::debug!("features={:#?}", r.activated_features);
         if r.opts.compare {
             r.compare();
@@ -386,11 +505,11 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     fn do_resolve(
         &mut self,
         specs: &[PackageIdSpec],
-        requested_features: &RequestedFeatures,
+        cli_features: &CliFeatures,
     ) -> CargoResult<()> {
-        let member_features = self.ws.members_with_features(specs, requested_features)?;
-        for (member, requested_features) in &member_features {
-            let fvs = self.fvs_from_requested(member.package_id(), requested_features);
+        let member_features = self.ws.members_with_features(specs, cli_features)?;
+        for (member, cli_features) in &member_features {
+            let fvs = self.fvs_from_requested(member.package_id(), cli_features);
             let for_host = self.track_for_host && self.is_proc_macro(member.package_id());
             self.activate_pkg(member.package_id(), for_host, &fvs)?;
             if for_host {
@@ -656,24 +775,19 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     fn fvs_from_requested(
         &self,
         pkg_id: PackageId,
-        requested_features: &RequestedFeatures,
+        cli_features: &CliFeatures,
     ) -> Vec<FeatureValue> {
         let summary = self.resolve.summary(pkg_id);
         let feature_map = summary.features();
-        if requested_features.all_features {
+        if cli_features.all_features {
             feature_map
                 .keys()
                 .map(|k| FeatureValue::Feature(*k))
                 .collect()
         } else {
-            let mut result: Vec<FeatureValue> = requested_features
-                .features
-                .as_ref()
-                .iter()
-                .map(|f| FeatureValue::new(*f))
-                .collect();
+            let mut result: Vec<FeatureValue> = cli_features.features.iter().cloned().collect();
             let default = InternedString::new("default");
-            if requested_features.uses_default_features && feature_map.contains_key(&default) {
+            if cli_features.uses_default_features && feature_map.contains_key(&default) {
                 result.push(FeatureValue::Feature(default));
             }
             result
