@@ -342,13 +342,11 @@ fn named_config_profile() {
     // foo -> middle -> bar -> dev
     // middle exists in Cargo.toml, the others in .cargo/config
     use super::config::ConfigBuilder;
-    use cargo::core::compiler::CompileMode;
-    use cargo::core::enable_nightly_features;
+    use cargo::core::compiler::{CompileKind, CompileMode};
     use cargo::core::profiles::{Profiles, UnitFor};
     use cargo::core::{PackageId, Workspace};
     use cargo::util::interning::InternedString;
     use std::fs;
-    enable_nightly_features();
     paths::root().join(".cargo").mkdir_p();
     fs::write(
         paths::root().join(".cargo/config"),
@@ -394,7 +392,7 @@ fn named_config_profile() {
         "#,
     )
     .unwrap();
-    let config = ConfigBuilder::new().build();
+    let config = ConfigBuilder::new().nightly_features_allowed(true).build();
     let profile_name = InternedString::new("foo");
     let ws = Workspace::new(&paths::root().join("Cargo.toml"), &config).unwrap();
     let profiles = Profiles::new(&ws, profile_name).unwrap();
@@ -405,7 +403,8 @@ fn named_config_profile() {
 
     // normal package
     let mode = CompileMode::Build;
-    let p = profiles.get_profile(a_pkg, true, true, UnitFor::new_normal(), mode);
+    let kind = CompileKind::Host;
+    let p = profiles.get_profile(a_pkg, true, true, UnitFor::new_normal(), mode, kind);
     assert_eq!(p.name, "foo");
     assert_eq!(p.codegen_units, Some(2)); // "foo" from config
     assert_eq!(p.opt_level, "1"); // "middle" from manifest
@@ -414,7 +413,7 @@ fn named_config_profile() {
     assert_eq!(p.overflow_checks, true); // "dev" built-in (ignore package override)
 
     // build-override
-    let bo = profiles.get_profile(a_pkg, true, true, UnitFor::new_host(false), mode);
+    let bo = profiles.get_profile(a_pkg, true, true, UnitFor::new_host(false), mode, kind);
     assert_eq!(bo.name, "foo");
     assert_eq!(bo.codegen_units, Some(6)); // "foo" build override from config
     assert_eq!(bo.opt_level, "0"); // default to zero
@@ -423,7 +422,7 @@ fn named_config_profile() {
     assert_eq!(bo.overflow_checks, true); // SAME as normal
 
     // package overrides
-    let po = profiles.get_profile(dep_pkg, false, true, UnitFor::new_normal(), mode);
+    let po = profiles.get_profile(dep_pkg, false, true, UnitFor::new_normal(), mode, kind);
     assert_eq!(po.name, "foo");
     assert_eq!(po.codegen_units, Some(7)); // "foo" package override from config
     assert_eq!(po.opt_level, "1"); // SAME as normal
